#!/usr/bin/perl -w

use strict;

my $pdb=$ARGV[0];
my $chainID=$ARGV[1];
my $separation=$ARGV[2];

my @lines=&extractChain($pdb,$chainID);
my @xyz=&genXYZ(@lines);

my @residues;
foreach(@xyz) {
	my @v=split(/,/);
	my $aa=shift @v;
	my %res;
	$res{aa}=$aa;
	$res{coord}=join(" ",@v);

	push @residues,\%res;
}

my $num=@residues;
open(FH,">tmpQFormat.dat") or die "Cannot create tmpQFormat.dat";
print FH "3\n";
print FH "$num\n";

foreach(@residues) {
	print FH $_->{coord}."\n";
}
close(FH);

`qhull d i TI tmpQFormat.dat TO tmp.dt`;

my @map;
for(my $i=0;$i<$num;$i++) {
	for(my $j=0;$j<$num;$j++) {
		$map[$i][$j]=0;
	}
}


open(FH,"tmp.dt") or die "Cannot open tmp.dt";
my $numCells=<FH>;
for(my $i=0;$i<$numCells;$i++) {
	my $line=<FH>;
	chomp $line;
	my @v=split(/ /,$line);
	my $numV=@v;
	for(my $j=0;$j<$numV-1;$j++) {
		for(my $k=$j+1;$k<$numV;$k++) {
			$map[$v[$j]][$v[$k]]=1;
			$map[$v[$k]][$v[$j]]=1;
		}
	}
}

for(my $i=0;$i<$num;$i++) {
        for(my $j=0;$j<=$i-$separation;$j++) {
                my $r1=$i+1;
                my $r2=$j+1;
		print "$r1,$r2,$map[$i][$j]\n";
        }
        for(my $j=$i+$separation;$j<$num;$j++) {
                my $r1=$i+1;
                my $r2=$j+1;
		print "$r1,$r2,$map[$i][$j]\n";
        }
}

unlink "tmpQFormat.dat";
unlink "tmp.dt";


sub extractChain {
        my $pdb=shift @_;
        my $chainID=shift @_;

        my @atoms;

        my @lines=`cat $pdb`;
        chomp @lines;
        $chainID=" " if(not defined $chainID);
        $chainID=" " if($chainID eq "_");

        my $lastResidue;
        foreach my $line (@lines) {
                if($line=~/^ATOM/) {
                        my $resName=substr($line,17,3);
                        my $atomName=substr($line,12,4);
                        $atomName=~s/ //g;
                        my $cID=substr($line,21,1);
                        my $residueID=substr($line,22,5);
                        if((($resName ne "GLY" and $atomName eq "CB") or ($resName eq "GLY" and $atomName eq "CA")) and $cID eq $chainID) {
                                next if(defined $lastResidue and $residueID eq $lastResidue);
                                $lastResidue=$residueID;
                                push @atoms,$line;
                        }
                }
                last if($line=~/^ENDMDL/);
        }

        return @atoms;
}

sub filterResName {
        my $name=shift @_;

        if($name eq "ALA") {
                return "A";
        } elsif($name eq "CYS") {
                return "C";
        } elsif($name eq "ASP") {
                return "D";
        } elsif($name eq "GLU") {
                return "E";
        } elsif($name eq "PHE") {
                return "F";
        } elsif($name eq "GLY") {
                return "G";
        } elsif($name eq "HIS") {
                return "H";
        } elsif($name eq "ILE") {
                return "I";
        } elsif($name eq "LYS") {
                return "K";
        } elsif($name eq "LEU") {
                return "L";
        } elsif($name eq "MET") {
                return "M";
        } elsif($name eq "ASN") {
                return "N";
        } elsif($name eq "PRO") {
                return "P";
        } elsif($name eq "GLN") {
                return "Q";
        } elsif($name eq "ARG") {
                return "R";
        } elsif($name eq "SER") {
                return "S";
        } elsif($name eq "THR") {
                return "T";
        } elsif($name eq "VAL") {
                return "V";
        } elsif($name eq "TRP") {
                return "W";
        } elsif($name eq "TYR") {
                return "Y";
        }

        return "X";
}



sub genXYZ {
	my @lines=@_;
	my @atoms;

	foreach my $line (@lines) {
	        if($line=~/^ATOM/) {
	                my %atom;
	                my $resName=&filterResName(substr($line,17,3));
	                my $x=substr($line,30,8);
	                $x=~s/ //g;
	                my $y=substr($line,38,8);
	                $y=~s/ //g;
	                my $z=substr($line,46,8);
	                $z=~s/ //g;
	                push @atoms,"$resName,$x,$y,$z";
        	}
	}

	return @atoms;
}
	
