#!/usr/bin/perl -w

use strict;
use Graph::Kruskal qw(:all);

my $pdb=$ARGV[0];
my $chainID=$ARGV[1];
my $separation=$ARGV[2];

my $rngScript="~/scripts/compute_RNG_map.pl";
my @mapRNG=`$rngScript $pdb $chainID $separation`;
chomp @mapRNG;

my @map;
foreach(@mapRNG) {
        my @v=split(/,/);
        my $r1=$v[0]-1;
        my $r2=$v[1]-1;
        $map[$r1][$r2]=$v[2];
}

my @lines=&extractChain($pdb,$chainID);
my @residues=&genXYZ(@lines);
my $num=@residues;

my @vert;
for(my $i=0;$i<$num;$i++) {
	push @vert,$i+1;
}

&define_vortices(@vert);

my @edges;
for(my $i=0;$i<$num-1;$i++) {
	for(my $j=$i+1;$j<$num;$j++) {
		if(defined $map[$i][$j] and $map[$i][$j]==1) {
			my $dist=&dist($residues[$i]->{coord},$residues[$j]->{coord});
			push @edges,($i+1,$j+1,$dist);
		}
	}
}

&define_edges(@edges);

my @MST=&kruskal();

for(my $i=0;$i<$num;$i++) {
	for(my $j=0;$j<$num;$j++) {
		$map[$i][$j]=0;
	}
}

foreach(@MST) {
	next if(not defined $_->{from} or not defined $_->{to});
	my $v1=$_->{from}-1;
	my $v2=$_->{to}-1;
	$map[$v1][$v2]=1;
	$map[$v2][$v1]=1;
}

for(my $i=0;$i<$num;$i++) {
        for(my $j=0;$j<=$i-$separation;$j++) {
                my $r1=$i+1;
                my $r2=$j+1;
                print "$r1,$r2,$map[$i][$j]\n";
        }
        for(my $j=$i+$separation;$j<$num;$j++) {
                my $r1=$i+1;
                my $r2=$j+1;
                print "$r1,$r2,$map[$i][$j]\n";
        }
}

sub dist {
	my $c1=shift @_;
	my $c2=shift @_;

	return sqrt(($c1->[0]-$c2->[0])**2+($c1->[1]-$c2->[1])**2+($c1->[2]-$c2->[2])**2);	
}

sub extractChain {
        my $pdb=shift @_;
        my $chainID=shift @_;

        my @atoms;

        my @lines=`cat $pdb`;
        chomp @lines;
        $chainID=" " if(not defined $chainID);
        $chainID=" " if($chainID eq "_");

        my $lastResidue;
        foreach my $line (@lines) {
                if($line=~/^ATOM/) {
                        my $resName=substr($line,17,3);
                        my $atomName=substr($line,12,4);
                        $atomName=~s/ //g;
                        my $cID=substr($line,21,1);
                        my $residueID=substr($line,22,5);
                        if((($resName ne "GLY" and $atomName eq "CB") or ($resName eq "GLY" and $atomName eq "CA")) and $cID eq $chainID) {
                                next if(defined $lastResidue and $residueID eq $lastResidue);
                                $lastResidue=$residueID;
                                push @atoms,$line;
                        }
                }
                last if($line=~/^ENDMDL/);
        }

        return @atoms;
}

sub filterResName {
        my $name=shift @_;

        if($name eq "ALA") {
                return "A";
        } elsif($name eq "CYS") {
                return "C";
        } elsif($name eq "ASP") {
                return "D";
        } elsif($name eq "GLU") {
                return "E";
        } elsif($name eq "PHE") {
                return "F";
        } elsif($name eq "GLY") {
                return "G";
        } elsif($name eq "HIS") {
                return "H";
        } elsif($name eq "ILE") {
                return "I";
        } elsif($name eq "LYS") {
                return "K";
        } elsif($name eq "LEU") {
                return "L";
        } elsif($name eq "MET") {
                return "M";
        } elsif($name eq "ASN") {
                return "N";
        } elsif($name eq "PRO") {
                return "P";
        } elsif($name eq "GLN") {
                return "Q";
        } elsif($name eq "ARG") {
                return "R";
        } elsif($name eq "SER") {
                return "S";
        } elsif($name eq "THR") {
                return "T";
        } elsif($name eq "VAL") {
                return "V";
        } elsif($name eq "TRP") {
                return "W";
        } elsif($name eq "TYR") {
                return "Y";
        }

        return "X";
}



sub genXYZ {
        my @lines=@_;
        my @atoms;

        foreach my $line (@lines) {
                if($line=~/^ATOM/) {
                        my %atom;
                        my $resName=&filterResName(substr($line,17,3));
                        my $x=substr($line,30,8);
                        $x=~s/ //g;
                        my $y=substr($line,38,8);
                        $y=~s/ //g;
                        my $z=substr($line,46,8);
                        $z=~s/ //g;
                        my %res;
                        $res{aa}=$resName;
                        my @coord=($x,$y,$z);
                        $res{coord}=\@coord;
                        push @atoms,\%res;
                }
        }

        return @atoms;
}
