#!/usr/bin/perl -w

use strict;

my $pdb=$ARGV[0];
my $chainID=$ARGV[1];

my @lines=&extractChain($pdb,$chainID);
chomp @lines;
my @atoms;
my @AAs;

my $tmpFile="tmpXYZ$$.dat";
open(FH,">$tmpFile") or die "Can't create $tmpFile";
foreach my $line (@lines) {
	my %atom;
	my $resName=&filterResName(substr($line,17,3));
	my $x=substr($line,30,8);
	$x=~s/ //g;
	my $y=substr($line,38,8);
	$y=~s/ //g;
	my $z=substr($line,46,8);
	$z=~s/ //g;
	print FH "$resName,$x,$y,$z\n";
	push @AAs,$resName,
}

close FH;


#Please change /usr/local/bin/ in the next line to where qhull is installed
my @rchStates=`./xyz2rch.exe /usr/local/bin/ $tmpFile`;
unlink $tmpFile;
chomp @rchStates;

my $num1=@AAs;
my $num2=@rchStates;

die "Integrity error" if($num1!=$num2);

for(my $i=0;$i<$num1;$i++) {
	print "$AAs[$i],$rchStates[$i]\n";
}



sub filterResName {
	my $name=shift @_;

        if($name eq "ALA") {
                return "A";
        } elsif($name eq "CYS") {
                return "C";
        } elsif($name eq "ASP") {
                return "D";
        } elsif($name eq "GLU") {
                return "E";
        } elsif($name eq "PHE") {
                return "F";
        } elsif($name eq "GLY") {
                return "G";
        } elsif($name eq "HIS") {
                return "H";
        } elsif($name eq "ILE") {
                return "I";
        } elsif($name eq "LYS") {
                return "K";
        } elsif($name eq "LEU") {
                return "L";
        } elsif($name eq "MET") {
                return "M";
        } elsif($name eq "ASN") {
                return "N";
        } elsif($name eq "PRO") {
                return "P";
        } elsif($name eq "GLN") {
                return "Q";
        } elsif($name eq "ARG") {
                return "R";
        } elsif($name eq "SER") {
                return "S";
        } elsif($name eq "THR") {
                return "T";
        } elsif($name eq "VAL") {
                return "V";
        } elsif($name eq "TRP") {
                return "W";
        } elsif($name eq "TYR") {
                return "Y";
	}

	return "X";
}

sub extractChain {
	my $pdb=shift @_;
	my $chainID=shift @_;

	my @atoms;

	my @lines=`cat $pdb`;
	chomp @lines;
	$chainID=" " if(not defined $chainID);
	$chainID=" " if($chainID eq "_");
	
	my $lastResidue;
	foreach my $line (@lines) {
		if($line=~/^ATOM/) {
			my $resName=substr($line,17,3);
			my $atomName=substr($line,12,4);
			$atomName=~s/ //g;
			my $cID=substr($line,21,1);
			my $residueID=substr($line,22,5);
			if((($resName ne "GLY" and $atomName eq "CB") or ($resName eq "GLY" and $atomName eq "CA")) and $cID eq $chainID) {
				next if(defined $lastResidue and $residueID eq $lastResidue);
				$lastResidue=$residueID;
				push @atoms,$line;
			}
		}
		last if($line=~/^ENDMDL/);
	}

	return @atoms;
}

