#!/usr/bin/perl -w

use strict;

my $pdb=$ARGV[0];
my $chainID=$ARGV[1];
my $separation=$ARGV[2];

my $ggScript="~/scripts/compute_GG_map.pl";
my @mapGG=`$ggScript $pdb $chainID $separation`;
chomp @mapGG;

my @map;
foreach(@mapGG) {
        my @v=split(/,/);
        my $r1=$v[0]-1;
        my $r2=$v[1]-1;
        $map[$r1][$r2]=$v[2];
}

my @lines=&extractChain($pdb,$chainID);
my @residues=&genXYZ(@lines);
my $num=@residues;

my @dist;
for(my $i=0;$i<$num-1;$i++) {
	for(my $j=$i+1;$j<$num;$j++) {
		my $dist=&dist($residues[$i]->{coord},$residues[$j]->{coord});
		$dist[$i][$j]=$dist;
		$dist[$j][$i]=$dist;
	}
}

for(my $i=0;$i<$num-1;$i++) {
	for(my $j=$i+1;$j<$num;$j++) {
		if(defined $map[$i][$j] and $map[$i][$j]==1) {
			for(my $k=0;$k<$num;$k++) {
				next if($k==$i or $k==$j);
				my $radius=$dist[$i][$j];
				if($dist[$i][$k]<$radius and $dist[$j][$k]<$radius) {
					#print "Discarded $i $j because of $k : $distI $distJ $radius\n";
					$map[$i][$j]=0;
					$map[$j][$i]=0;
					last;
				}
			}
		}
	}
}

for(my $i=0;$i<$num;$i++) {
        for(my $j=0;$j<$num;$j++) {
                if(defined $map[$i][$j]) {
                        my $r1=$i+1;
                        my $r2=$j+1;
                        print "$r1,$r2,$map[$i][$j]\n";
                }
        }
}

sub computeCR {
	my $c1=shift @_;
	my $c2=shift @_;

	my @center;
	$center[0]=($c1->[0]+$c2->[0])/2.0;
	$center[1]=($c1->[1]+$c2->[1])/2.0;
	$center[2]=($c1->[2]+$c2->[2])/2.0;
	my $radius=&dist($c1,$c2)/2;

	return (\@center,$radius);
}

sub dist {
	my $c1=shift @_;
	my $c2=shift @_;

	return sqrt(($c1->[0]-$c2->[0])**2+($c1->[1]-$c2->[1])**2+($c1->[2]-$c2->[2])**2);	
}

sub extractChain {
        my $pdb=shift @_;
        my $chainID=shift @_;

        my @atoms;

        my @lines=`cat $pdb`;
        chomp @lines;
        $chainID=" " if(not defined $chainID);
        $chainID=" " if($chainID eq "_");

        my $lastResidue;
        foreach my $line (@lines) {
                if($line=~/^ATOM/) {
                        my $resName=substr($line,17,3);
                        my $atomName=substr($line,12,4);
                        $atomName=~s/ //g;
                        my $cID=substr($line,21,1);
                        my $residueID=substr($line,22,5);
                        if((($resName ne "GLY" and $atomName eq "CB") or ($resName eq "GLY" and $atomName eq "CA")) and $cID eq $chainID) {
                                next if(defined $lastResidue and $residueID eq $lastResidue);
                                $lastResidue=$residueID;
                                push @atoms,$line;
                        }
                }
                last if($line=~/^ENDMDL/);
        }

        return @atoms;
}

sub filterResName {
        my $name=shift @_;

        if($name eq "ALA") {
                return "A";
        } elsif($name eq "CYS") {
                return "C";
        } elsif($name eq "ASP") {
                return "D";
        } elsif($name eq "GLU") {
                return "E";
        } elsif($name eq "PHE") {
                return "F";
        } elsif($name eq "GLY") {
                return "G";
        } elsif($name eq "HIS") {
                return "H";
        } elsif($name eq "ILE") {
                return "I";
        } elsif($name eq "LYS") {
                return "K";
        } elsif($name eq "LEU") {
                return "L";
        } elsif($name eq "MET") {
                return "M";
        } elsif($name eq "ASN") {
                return "N";
        } elsif($name eq "PRO") {
                return "P";
        } elsif($name eq "GLN") {
                return "Q";
        } elsif($name eq "ARG") {
                return "R";
        } elsif($name eq "SER") {
                return "S";
        } elsif($name eq "THR") {
                return "T";
        } elsif($name eq "VAL") {
                return "V";
        } elsif($name eq "TRP") {
                return "W";
        } elsif($name eq "TYR") {
                return "Y";
        }

        return "X";
}



sub genXYZ {
        my @lines=@_;
        my @atoms;

        foreach my $line (@lines) {
                if($line=~/^ATOM/) {
                        my %atom;
                        my $resName=&filterResName(substr($line,17,3));
                        my $x=substr($line,30,8);
                        $x=~s/ //g;
                        my $y=substr($line,38,8);
                        $y=~s/ //g;
                        my $z=substr($line,46,8);
                        $z=~s/ //g;
                        my %res;
                        $res{aa}=$resName;
                        my @coord=($x,$y,$z);
                        $res{coord}=\@coord;
                        push @atoms,\%res;
                }
        }

        return @atoms;
}
