#!/usr/bin/perl -w

use strict;

my $pdb=$ARGV[0];
my $chainID=$ARGV[1];
my $cutOff=$ARGV[2];
my $separation=$ARGV[3];
$separation=6 if(not defined $separation);

my @lines=&extractChain($pdb,$chainID);
chomp @lines;
my @atoms;

foreach my $line (@lines) {
	my %atom;
	$atom{resName}=&filterResName(substr($line,17,3));
	$atom{x}=substr($line,30,8);
	$atom{x}=~s/ //g;
	$atom{y}=substr($line,38,8);
	$atom{y}=~s/ //g;
	$atom{z}=substr($line,46,8);
	$atom{z}=~s/ //g;
	push @atoms,\%atom;
}

my ($i,$j);

for(my $i=0;$i<@atoms;$i++) {
	my $cn=0;
	for(my $j=0;$j<$i-$separation;$j++) {
		my $dist=&computeDistance($atoms[$i],$atoms[$j]);
		$cn++ if($dist<$cutOff);
	}
	for(my $j=$i+$separation+1;$j<@atoms;$j++) {
		my $dist=&computeDistance($atoms[$i],$atoms[$j]);
		$cn++ if($dist<$cutOff);
	}
	print "$atoms[$i]->{resName},$cn\n";
}

sub computeDistance {
	my $atomA=shift @_;
	my $atomB=shift @_;

	return sqrt(($atomA->{x}-$atomB->{x})**2
		+($atomA->{y}-$atomB->{y})**2
		+($atomA->{z}-$atomB->{z})**2);
}

sub filterResName {
	my $name=shift @_;

        if($name eq "ALA") {
                return "A";
        } elsif($name eq "CYS") {
                return "C";
        } elsif($name eq "ASP") {
                return "D";
        } elsif($name eq "GLU") {
                return "E";
        } elsif($name eq "PHE") {
                return "F";
        } elsif($name eq "GLY") {
                return "G";
        } elsif($name eq "HIS") {
                return "H";
        } elsif($name eq "ILE") {
                return "I";
        } elsif($name eq "LYS") {
                return "K";
        } elsif($name eq "LEU") {
                return "L";
        } elsif($name eq "MET") {
                return "M";
        } elsif($name eq "ASN") {
                return "N";
        } elsif($name eq "PRO") {
                return "P";
        } elsif($name eq "GLN") {
                return "Q";
        } elsif($name eq "ARG") {
                return "R";
        } elsif($name eq "SER") {
                return "S";
        } elsif($name eq "THR") {
                return "T";
        } elsif($name eq "VAL") {
                return "V";
        } elsif($name eq "TRP") {
                return "W";
        } elsif($name eq "TYR") {
                return "Y";
	}

	return "X";
}

sub extractChain {
	my $pdb=shift @_;
	my $chainID=shift @_;

	my @atoms;

	my @lines=`cat $pdb`;
	chomp @lines;
	$chainID=" " if(not defined $chainID);
	$chainID=" " if($chainID eq "_");
	
	my $lastResidue;
	foreach my $line (@lines) {
		if($line=~/^ATOM/) {
			my $resName=substr($line,17,3);
			my $atomName=substr($line,12,4);
			$atomName=~s/ //g;
			my $cID=substr($line,21,1);
			my $residueID=substr($line,22,5);
			if((($resName ne "GLY" and $atomName eq "CB") or ($resName eq "GLY" and $atomName eq "CA")) and $cID eq $chainID) {
				next if(defined $lastResidue and $residueID eq $lastResidue);
				$lastResidue=$residueID;
				push @atoms,$line;
			}
		}
		last if($line=~/^ENDMDL/);
	}

	return @atoms;
}
			
